@@ -194,9 +194,9 @@ void MhwInterfaces::Destroy()
     {
         return;
     }
-    if (m_miInterface && m_miInterface->GetOsInterface())
+    if (m_osInterface)
     {
-        m_miInterface->GetOsInterface()->pfnDeleteMhwCpInterface(m_cpInterface);
+        m_osInterface->pfnDeleteMhwCpInterface(m_cpInterface);
         m_cpInterface = nullptr;
     }
     else
@@ -375,10 +375,16 @@ void* MmdDevice::CreateFactory(
     {
         MMD_FAILURE();
     }
+
+    // transfer ownership of osinterface. No need to delete osinterface from this point.
     device->Initialize(osInterface, mhwInterfaces);
     if (device->m_mmdDevice == nullptr)
     {
-        MMD_FAILURE();
+        mhwInterfaces->Destroy();
+        // no need to delete osinterface becauses it's already deleted in device->Initialize
+        MOS_Delete(mhwInterfaces);
+        MOS_Delete(device);
+        return nullptr;
     }
 
     void *mmdDevice = device->m_mmdDevice;
@@ -403,25 +409,23 @@ MhwInterfaces* MmdDevice::CreateMhwInterface(PMOS_INTERFACE osInterface)
 void* McpyDevice::CreateFactory(
     PMOS_CONTEXT    osDriverContext)
 {
-#define MCPY_FAILURE()                                       \
-{                                                           \
-    if (mhwInterfaces != nullptr)                           \
-    {                                                       \
-        mhwInterfaces->Destroy();                           \
-    }                                                       \
-    MOS_Delete(mhwInterfaces);                              \
-    if (osInterface != nullptr)                             \
-    {                                                       \
-        if (osInterface->pfnDestroy)                        \
-        {                                                   \
-            osInterface->pfnDestroy(osInterface, false);    \
-        }                                                   \
-        MOS_FreeMemory(osInterface);                        \
-    }                                                       \
-    MOS_Delete(device);                                     \
-    return nullptr;                                         \
-}
     MHW_FUNCTION_ENTER;
+    
+    PMOS_INTERFACE osInterface   = nullptr;
+    MhwInterfaces *mhwInterfaces = nullptr;
+    McpyDevice     *device       = nullptr;
+
+    auto deleterOnFailure = [&](bool deleteOsInterface){
+        if (deleteOsInterface && osInterface != nullptr)
+        {
+            if (osInterface->pfnDestroy)
+            {
+                osInterface->pfnDestroy(osInterface, false);
+            }
+            MOS_FreeMemory(osInterface);
+        }
+        MOS_Delete(device);
+    };
 
     if (osDriverContext == nullptr)
     {
@@ -429,10 +433,6 @@ void* McpyDevice::CreateFactory(
         return nullptr;
     }
 
-    PMOS_INTERFACE osInterface   = nullptr;
-    MhwInterfaces *mhwInterfaces = nullptr;
-    McpyDevice     *device       = nullptr;
-
     osInterface = (PMOS_INTERFACE)MOS_AllocAndZeroMemory(sizeof(MOS_INTERFACE));
     if (osInterface == nullptr)
     {
@@ -443,7 +443,8 @@ void* McpyDevice::CreateFactory(
         osDriverContext,
         COMPONENT_MCPY) != MOS_STATUS_SUCCESS)
     {
-        MCPY_FAILURE();
+        deleterOnFailure(true);
+        return nullptr;
     }
 
     PLATFORM platform = {};
@@ -451,18 +452,16 @@ void* McpyDevice::CreateFactory(
     device = McpyFactory::Create(platform.eProductFamily);
     if (device == nullptr)
     {
-        MCPY_FAILURE();
+        deleterOnFailure(true);
+        return nullptr;
     }
 
-    mhwInterfaces = device->CreateMhwInterface(osInterface);
-    if (mhwInterfaces == nullptr)
-    {
-        MCPY_FAILURE();
-    }
-    device->Initialize(osInterface, mhwInterfaces);
+    // transfer ownership of osInterface. No need to delete it if init fails
+    device->Initialize(osInterface);
     if (device->m_mcpyDevice == nullptr)
     {
-        MCPY_FAILURE();
+        deleterOnFailure(false);
+        return nullptr;
     }
 
     void *mcpyDevice = device->m_mcpyDevice;
