@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2017-2022, Intel Corporation
+* Copyright (c) 2017-2024, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -160,6 +160,7 @@ MOS_STATUS MhwInterfacesG12Tgllp::Initialize(
         MHW_ASSERTMESSAGE("The OS interface is not valid!");
         return MOS_STATUS_INVALID_PARAMETER;
     }
+    m_osInterface = osInterface;
 
     auto gtSystemInfo = osInterface->pfnGetGtSystemInfo(osInterface);
     if (gtSystemInfo == nullptr)
@@ -408,48 +409,52 @@ static bool tgllpRegisteredMcpy =
     Register<McpyDeviceG12Tgllp>((uint32_t)IGFX_TIGERLAKE_LP);
 
 MOS_STATUS McpyDeviceG12Tgllp::Initialize(
-    PMOS_INTERFACE osInterface,
-    MhwInterfaces *mhwInterfaces)
+    PMOS_INTERFACE osInterface)
 {
-#define MCPY_FAILURE()                                       \
-{                                                           \
-    if (device != nullptr)                                  \
-    {                                                       \
-        MOS_Delete(device);                                 \
-    }                                                       \
-    return MOS_STATUS_NO_SPACE;                             \
-}
-
     MHW_FUNCTION_ENTER;
 
     Mcpy *device = nullptr;
+    MhwInterfaces* mhwInterfaces = nullptr;
 
-    if (mhwInterfaces->m_miInterface == nullptr)
-    {
-        MCPY_FAILURE();
-    }
+    auto deleterOnFailure = [&](bool deleteOsInterface, bool deleteMhwInterface){
+        if (deleteOsInterface && osInterface != nullptr)
+        {
+            if (osInterface->pfnDestroy)
+            {
+                osInterface->pfnDestroy(osInterface, false);
+            }
+            MOS_FreeMemory(osInterface);
+        }
 
-    if (mhwInterfaces->m_veboxInterface == nullptr)
-    {
-        MCPY_FAILURE();
-    }
+        if (deleteMhwInterface && mhwInterfaces != nullptr)
+        {
+            mhwInterfaces->Destroy();
+            MOS_Delete(mhwInterfaces);
+        }
 
-    if (mhwInterfaces->m_bltInterface == nullptr)
-    {
-        MCPY_FAILURE();
-    }
+        MOS_Delete(device);
+    };
 
     device = MOS_New(Mcpy);
-
     if (device == nullptr)
     {
-        MCPY_FAILURE();
+        deleterOnFailure(true, false);
+        return MOS_STATUS_NO_SPACE;
+    }
+
+    mhwInterfaces = CreateMhwInterface(osInterface);
+    if (mhwInterfaces->m_miInterface == nullptr ||
+        mhwInterfaces->m_veboxInterface == nullptr ||
+        mhwInterfaces->m_bltInterface == nullptr)
+    {
+        deleterOnFailure(true, true);
+        return MOS_STATUS_NO_SPACE;
     }
 
     if (device->Initialize(
         osInterface, mhwInterfaces) != MOS_STATUS_SUCCESS)
     {
-        MOS_Delete(device);
+        deleterOnFailure(false, false);
         MOS_OS_CHK_STATUS_RETURN(MOS_STATUS_UNINITIALIZED);
     }
 
@@ -525,7 +530,6 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     if (hwInterface->m_hwInterfaceNext == nullptr)
     {
         MOS_Delete(hwInterface);
-        mhwInterfaces->SetDestroyState(true);
         CODECHAL_PUBLIC_ASSERTMESSAGE("hwInterfaceNext is not valid!");
         return MOS_STATUS_NO_SPACE;
     }
@@ -538,7 +542,6 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     if (debugInterface == nullptr)
     {
         MOS_Delete(hwInterface);
-        mhwInterfaces->SetDestroyState(true);
         CODECHAL_PUBLIC_ASSERTMESSAGE("debugInterface is not valid!");
         return MOS_STATUS_NO_SPACE;
     }
@@ -554,13 +557,21 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     CodechalDebugInterface *debugInterface = nullptr;
 #endif // USE_CODECHAL_DEBUG_TOOL
 
+    auto release_func = [&]() 
+    {
+        MOS_Delete(hwInterface);
+#if USE_CODECHAL_DEBUG_TOOL
+        MOS_Delete(debugInterface);
+#endif  // USE_CODECHAL_DEBUG_TOOL
+    };
+
     if (CodecHalIsDecode(CodecFunction))
     {
     #ifdef _MPEG2_DECODE_SUPPORTED
         if (info->Mode == CODECHAL_DECODE_MODE_MPEG2IDCT ||
             info->Mode == CODECHAL_DECODE_MODE_MPEG2VLD)
         {
-        #ifdef _APOGEIOS_SUPPORTED
+        #ifdef _MEDIA_RESERVED
             bool apogeiosEnable = false;
             MOS_USER_FEATURE_VALUE_DATA         userFeatureData;
             MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
@@ -598,7 +609,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
         if (info->Mode == CODECHAL_DECODE_MODE_AVCVLD)
         {
             bool apogeiosEnable = false;
-        #ifdef _APOGEIOS_SUPPORTED
+        #ifdef _MEDIA_RESERVED
             MOS_USER_FEATURE_VALUE_DATA         userFeatureData;
             MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
 
@@ -624,7 +635,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             if (m_codechalDevice == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Failed to create decode device!");
-                return MOS_STATUS_NO_SPACE;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NO_SPACE, release_func);
             }
  #ifdef _DECODE_PROCESSING_SUPPORTED
 
@@ -638,14 +649,14 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
                 if (decoder == nullptr)
                 {
                     CODECHAL_PUBLIC_ASSERTMESSAGE("Failed to create decode device!");
-                    return MOS_STATUS_NO_SPACE;
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NO_SPACE, release_func);
                 }
                 FieldScalingInterface *fieldScalingInterface =
                     MOS_New(Decode::FieldScaling, hwInterface);
                 if (fieldScalingInterface == nullptr)
                 {
                     CODECHAL_PUBLIC_ASSERTMESSAGE("Failed to create field scaling interface!");
-                    return MOS_STATUS_NO_SPACE;
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NO_SPACE, release_func);
                 }
                 decoder->m_fieldScalingInterface = fieldScalingInterface;
             }
@@ -657,7 +668,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
         if (info->Mode == CODECHAL_DECODE_MODE_JPEG)
         {
             bool apogeiosEnable = false;
-#ifdef _APOGEIOS_SUPPORTED
+#ifdef _MEDIA_RESERVED
             MOS_USER_FEATURE_VALUE_DATA userFeatureData;
             MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
 
@@ -692,7 +703,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     #ifdef _HEVC_DECODE_SUPPORTED
         if (info->Mode == CODECHAL_DECODE_MODE_HEVCVLD)
         {
-            #ifdef _APOGEIOS_SUPPORTED
+            #ifdef _MEDIA_RESERVED
             bool apogeiosEnable = false;
             MOS_USER_FEATURE_VALUE_DATA         userFeatureData;
             MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
@@ -721,7 +732,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     #ifdef _VP9_DECODE_SUPPORTED
         if (info->Mode == CODECHAL_DECODE_MODE_VP9VLD)
         {
-#ifdef _APOGEIOS_SUPPORTED
+#ifdef _MEDIA_RESERVED
             bool                        apogeiosEnable = false;
             MOS_USER_FEATURE_VALUE_DATA userFeatureData;
             MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
@@ -757,18 +768,13 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     #endif
         {
             CODECHAL_PUBLIC_ASSERTMESSAGE("Decode mode requested invalid!");
-            return MOS_STATUS_INVALID_PARAMETER;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
         }
 
         if (m_codechalDevice == nullptr)
         {
-            MOS_Delete(hwInterface);
-            mhwInterfaces->SetDestroyState(true);
-#if USE_CODECHAL_DEBUG_TOOL
-            MOS_Delete(debugInterface);
-#endif
             CODECHAL_PUBLIC_ASSERTMESSAGE("Decoder device creation failed!");
-            return MOS_STATUS_NO_SPACE;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_NO_SPACE, release_func);
         }
     }
     else if (CodecHalIsEncode(CodecFunction))
@@ -803,7 +809,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -819,7 +825,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -836,7 +842,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode allocation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -855,7 +861,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -891,7 +897,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             if (encoder == nullptr)
             {
                 CODECHAL_PUBLIC_ASSERTMESSAGE("Encode state creation failed!");
-                return MOS_STATUS_INVALID_PARAMETER;
+                CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
             }
             else
             {
@@ -905,7 +911,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
 #endif
         {
             CODECHAL_PUBLIC_ASSERTMESSAGE("Unsupported encode function requested.");
-            return MOS_STATUS_INVALID_PARAMETER;
+            CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
         }
 #if defined(ENABLE_KERNELS) && !defined(_FULL_OPEN_SOURCE)
         if (info->Mode != CODECHAL_ENCODE_MODE_JPEG)
@@ -915,7 +921,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
             {
                 if ((encoder->m_cscDsState = MOS_New(Encode::CscDsMdf, encoder)) == nullptr)
                 {
-                    return MOS_STATUS_INVALID_PARAMETER;
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
                 }
             }
             else
@@ -923,7 +929,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
                 // Create CSC and Downscaling interface
                 if ((encoder->m_cscDsState = MOS_New(Encode::CscDs, encoder)) == nullptr)
                 {
-                    return MOS_STATUS_INVALID_PARAMETER;
+                    CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
                 }
             }
         }
@@ -932,7 +938,7 @@ MOS_STATUS CodechalInterfacesG12Tgllp::Initialize(
     else
     {
         CODECHAL_PUBLIC_ASSERTMESSAGE("Unsupported codec function requested.");
-        return MOS_STATUS_INVALID_PARAMETER;
+        CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(MOS_STATUS_INVALID_PARAMETER, release_func);
     }
 
     return MOS_STATUS_SUCCESS;
