@@ -125,6 +125,7 @@ inline SurfaceFormat MosFormatToVdencSurfaceReconFormat(MOS_FORMAT format)
         return SurfaceFormat::p010Variant;
     case Format_R10G10B10A2:
         return SurfaceFormat::r10g10b10a2;
+    case Format_Y210:
     case Format_Y216:
         return SurfaceFormat::y216Variant;
     case Format_Y410:
@@ -143,6 +144,17 @@ class Impl : public Itf, public mhw::Impl
     MmioRegistersVdbox m_mmioRegisters[MHW_VDBOX_NODE_MAX] = {};  //!< Mfx mmio registers
 
 public:
+    virtual uint32_t GetCmd1CommandSize() override
+    {
+        // Just return 0 here, please implement logic in platform sepecific impl class.
+        return 0;
+    }
+
+    virtual uint32_t GetCmd2CommandSize() override
+    {
+        // Just return 0 here, please implement logic in platform sepecific impl class.
+        return 0;
+    }
 
     MOS_STATUS SetRowstoreCachingOffsets(const RowStorePar &par) override
     {
@@ -1145,7 +1157,7 @@ protected:
     DO_FIELD(DW6, TileStreaminOffset, params.tileStreamInOffset);                                                         \
                                                                                                                           \
     DO_FIELD(DW7, RowStoreOffsetEnable, cmd.DW4.TileStartCtbY == 0 ? params.tileEnable : 0);                              \
-    DO_FIELD(DW7, TileRowstoreOffset, cmd.DW4.TileStartCtbY == 0 ? cmd.DW4.TileStartCtbX / 32 : 0);                       \
+    DO_FIELD(DW7, TileRowstoreOffset, params.tileRowstoreOffset);                                                         \
                                                                                                                           \
     DO_FIELD(DW8, TileStreamoutOffsetEnable, params.tileEnable);                                                          \
     DO_FIELD(DW8, TileStreamoutOffset, params.tileId * 19);                                                               \
