@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2019-2021, Intel Corporation
+* Copyright (c) 2019-2024, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,7 @@
 using namespace vp;
 
 template <typename T>
-inline void swap(T &a, T &b)
+inline void _swap(T &a, T &b)
 {
     T tmp = b;
     b     = a;
@@ -135,6 +135,13 @@ void SwFilter::DestroySwFilter(SwFilter* p)
     }
 }
 
+VP_MHWINTERFACE *SwFilter::GetHwInterface()
+{
+    VP_FUNC_CALL();
+
+    return m_vpInterface.GetHwInterface();
+}
+
 /****************************************************************************************************/
 /*                                      SwFilterCsc                                                 */
 /****************************************************************************************************/
@@ -174,6 +181,7 @@ MOS_STATUS SwFilterCsc::Configure(VP_PIPELINE_PARAMS &params, bool isInputSurf,
     m_Params.output.chromaSiting    = surfOutput->ChromaSiting;
     m_Params.input.tileMode         = surfInput->TileModeGMM;
     m_Params.output.tileMode        = surfOutput->TileModeGMM;
+    m_Params.isFullRgbG10P709       = surfOutput->pGamutParams ? surfOutput->pGamutParams->GammaValue == GAMMA_1P0 : 0 && surfOutput->ColorSpace == CSpace_sRGB && IS_RGB64_FLOAT_FORMAT(surfOutput->Format);
     // Alpha should be handled in input pipe to avoid alpha data lost from image.
     m_Params.pAlphaParams           = params.pCompAlpha;
     // formatForCUS will be set on demand in Policy::GetCSCExecutionCapsBT2020ToRGB.
@@ -187,7 +195,7 @@ MOS_STATUS SwFilterCsc::Configure(VP_PIPELINE_PARAMS &params, bool isInputSurf,
 namespace vp
 {
 MOS_STATUS GetVeboxOutputParams(VP_EXECUTE_CAPS &executeCaps, MOS_FORMAT inputFormat, MOS_TILE_TYPE inputTileType, MOS_FORMAT outputFormat,
-                                MOS_FORMAT &veboxOutputFormat, MOS_TILE_TYPE &veboxOutputTileType);
+                                MOS_FORMAT &veboxOutputFormat, MOS_TILE_TYPE &veboxOutputTileType, VPHAL_CSPACE colorSpaceOutput);
 }
 
 MOS_STATUS SwFilterCsc::Configure(PVP_SURFACE surfInput, PVP_SURFACE surfOutput, VP_EXECUTE_CAPS caps)
@@ -205,7 +213,7 @@ MOS_STATUS SwFilterCsc::Configure(PVP_SURFACE surfInput, PVP_SURFACE surfOutput,
         MOS_TILE_TYPE   veboxOutputTileType = surfInput->osSurface->TileType;
 
         GetVeboxOutputParams(caps, surfInput->osSurface->Format, surfInput->osSurface->TileType,
-                            surfOutput->osSurface->Format, veboxOutputFormat, veboxOutputTileType);
+                            surfOutput->osSurface->Format, veboxOutputFormat, veboxOutputTileType, surfOutput->ColorSpace);
         m_Params.input.colorSpace = surfInput->ColorSpace;
         m_Params.output.colorSpace = surfInput->ColorSpace;
 
@@ -395,6 +403,7 @@ MOS_STATUS SwFilterScaling::Configure(VP_PIPELINE_PARAMS &params, bool isInputSu
         m_Params.rotation.rotationNeeded    = false;
         m_Params.output.dwWidth             = surfOutput->dwWidth;
         m_Params.output.dwHeight            = surfOutput->dwHeight;
+        m_Params.output.dwPitch             = surfOutput->dwPitch;
         m_Params.input.rcDst                = surfInput->rcDst;
         m_Params.output.rcSrc               = surfOutput->rcSrc;
         m_Params.output.rcDst               = surfOutput->rcDst;
@@ -405,6 +414,7 @@ MOS_STATUS SwFilterScaling::Configure(VP_PIPELINE_PARAMS &params, bool isInputSu
         m_Params.rotation.rotationNeeded    = true;
         m_Params.output.dwWidth             = surfOutput->dwHeight;
         m_Params.output.dwHeight            = surfOutput->dwWidth;
+        m_Params.output.dwPitch             = surfOutput->dwPitch;
         RECT_ROTATE(m_Params.input.rcDst, surfInput->rcDst);
         RECT_ROTATE(m_Params.output.rcSrc, surfOutput->rcSrc);
         RECT_ROTATE(m_Params.output.rcDst, surfOutput->rcDst);
@@ -496,6 +506,7 @@ MOS_STATUS SwFilterScaling::Configure(VEBOX_SFC_PARAMS &params)
         m_Params.rotation.rotationNeeded = false;
         m_Params.output.dwWidth     = params.output.surface->dwWidth;
         m_Params.output.dwHeight    = params.output.surface->dwHeight;
+        m_Params.output.dwPitch     = params.output.surface->dwPitch;
         m_Params.input.rcDst        = params.output.rcDst;
         m_Params.output.rcSrc       = recOutput;
         m_Params.output.rcDst       = recOutput;
@@ -506,6 +517,7 @@ MOS_STATUS SwFilterScaling::Configure(VEBOX_SFC_PARAMS &params)
         m_Params.rotation.rotationNeeded = true;
         m_Params.output.dwWidth     = params.output.surface->dwHeight;
         m_Params.output.dwHeight    = params.output.surface->dwWidth;
+        m_Params.output.dwPitch     = params.output.surface->dwPitch;
 
         RECT_ROTATE(m_Params.input.rcDst, params.output.rcDst);
         RECT_ROTATE(m_Params.output.rcSrc, recOutput);
@@ -544,6 +556,7 @@ MOS_STATUS SwFilterScaling::Configure(PVP_SURFACE surfInput, PVP_SURFACE surfOut
     m_Params.rotation.rotationNeeded    = false;
     m_Params.output.dwWidth             = surfOutput->osSurface->dwWidth;
     m_Params.output.dwHeight            = surfOutput->osSurface->dwHeight;
+    m_Params.output.dwPitch             = surfOutput->osSurface->dwPitch;
     m_Params.output.rcSrc               = surfOutput->rcSrc;
     m_Params.output.rcDst               = surfOutput->rcDst;
     m_Params.output.rcMaxSrc            = surfOutput->rcMaxSrc;
@@ -652,7 +665,7 @@ MOS_STATUS SwFilterScaling::Update(VP_SURFACE *inputSurf, VP_SURFACE *outputSurf
             // For VE-SFC + FC case, scaling is done by sfc, but scaling filter is still needed
             // by render workload for composition position.
             VP_PUBLIC_NORMALMESSAGE("Rotation has been done in execute pipe. Update scaling parameters.");
-            swap(m_Params.input.dwWidth, m_Params.input.dwHeight);
+            _swap(m_Params.input.dwWidth, m_Params.input.dwHeight);
             RECT tmp = m_Params.input.rcSrc;
             RECT_ROTATE(m_Params.input.rcSrc, tmp);
             tmp = m_Params.input.rcDst;
@@ -660,7 +673,7 @@ MOS_STATUS SwFilterScaling::Update(VP_SURFACE *inputSurf, VP_SURFACE *outputSurf
             tmp = m_Params.input.rcMaxSrc;
             RECT_ROTATE(m_Params.input.rcMaxSrc, tmp);
 
-            swap(m_Params.output.dwWidth, m_Params.output.dwHeight);
+            _swap(m_Params.output.dwWidth, m_Params.output.dwHeight);
             tmp = m_Params.output.rcSrc;
             RECT_ROTATE(m_Params.output.rcSrc, tmp);
             tmp = m_Params.output.rcDst;
@@ -920,6 +933,7 @@ MOS_STATUS SwFilterDeinterlace::Clean()
     VP_FUNC_CALL();
 
     VP_PUBLIC_CHK_STATUS_RETURN(SwFilter::Clean());
+    MOS_ZeroMemory(&m_Params, sizeof(m_Params));
     return MOS_STATUS_SUCCESS;
 }
 
@@ -931,6 +945,8 @@ MOS_STATUS SwFilterDeinterlace::Configure(VP_PIPELINE_PARAMS& params, bool isInp
     VP_PUBLIC_CHK_NULL_RETURN(surfInput);
     VP_PUBLIC_CHK_NULL_RETURN(surfInput->pDeinterlaceParams);
 
+    MOS_ZeroMemory(&m_Params, sizeof(m_Params));
+
     m_Params.formatInput          = surfInput->Format;
     m_Params.formatOutput         = surfInput->Format;
     m_Params.sampleTypeInput      = surfInput->SampleType;
@@ -1294,10 +1310,21 @@ MOS_STATUS SwFilterHdr::Configure(VP_PIPELINE_PARAMS &params, bool isInputSurf,
     uint32_t     dwUpdateMask = 0;
     bool                        bSupported       = false;
     VPHAL_HDR_LUT_MODE          CurrentLUTMode      = VPHAL_HDR_LUT_MODE_NONE;
+    VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface());
+    VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_userFeatureControl);
+    VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_vpPlatformInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_reporting);
+
+    auto userFeatureControl = m_vpInterface.GetHwInterface()->m_userFeatureControl;
+    auto vpPlatformInterface = m_vpInterface.GetHwInterface()->m_vpPlatformInterface;
+    VpFeatureReport *vpFeatureReport  = dynamic_cast<VpFeatureReport *>(m_vpInterface.GetHwInterface()->m_reporting);
+    m_Params.isOclKernelEnabled          = userFeatureControl->EnableOcl3DLut();
+#if (_DEBUG || _RELEASE_INTERNAL)
+    vpFeatureReport->GetFeatures().isOcl3DLut = m_Params.isOclKernelEnabled;
+#endif
 
     VP_PUBLIC_CHK_NULL_RETURN(surfInput);
     VP_PUBLIC_CHK_NULL_RETURN(surfOutput);
-    VP_PUBLIC_CHK_NULL_RETURN(surfInput->pHDRParams);
     VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface());
     VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_osInterface);
 
@@ -1305,34 +1332,36 @@ MOS_STATUS SwFilterHdr::Configure(VP_PIPELINE_PARAMS &params, bool isInputSurf,
     m_Params.formatOutput = surfOutput->Format;
     m_Params.widthInput   = surfInput->dwWidth;
     m_Params.heightInput  = surfInput->dwHeight;
+    if (surfInput->p3DLutParams)
+    {
+        m_Params.external3DLutParams = surfInput->p3DLutParams;
+    }
 
     // For H2S, it is possible that there is no HDR params for render target.
-    m_Params.uiMaxContentLevelLum = surfInput->pHDRParams->MaxCLL;
+    m_Params.uiMaxContentLevelLum = 4000;
     m_Params.srcColorSpace        = surfInput->ColorSpace;
     m_Params.dstColorSpace        = surfOutput->ColorSpace;
 
-    if (surfInput->pHDRParams->EOTF == VPHAL_HDR_EOTF_SMPTE_ST2084 ||
-       (surfInput->pHDRParams->EOTF == VPHAL_HDR_EOTF_TRADITIONAL_GAMMA_SDR && IS_RGB64_FLOAT_FORMAT(surfInput->Format))) // For FP16 HDR CSC typical usage
+    if (surfInput->pHDRParams)
     {
-        m_Params.hdrMode = VPHAL_HDR_MODE_TONE_MAPPING;
-        if (surfOutput->pHDRParams)
+        m_Params.uiMaxContentLevelLum = surfInput->pHDRParams->MaxCLL;
+        if (surfInput->pHDRParams->EOTF == VPHAL_HDR_EOTF_SMPTE_ST2084 ||
+           (surfInput->pHDRParams->EOTF == VPHAL_HDR_EOTF_TRADITIONAL_GAMMA_SDR && IS_RGB64_FLOAT_FORMAT(surfInput->Format))) // For FP16 HDR CSC typical usage
         {
-            m_Params.uiMaxDisplayLum = surfOutput->pHDRParams->max_display_mastering_luminance;
-            if (surfOutput->pHDRParams->EOTF == VPHAL_HDR_EOTF_SMPTE_ST2084)
+            m_Params.hdrMode = VPHAL_HDR_MODE_TONE_MAPPING;
+            if (surfOutput->pHDRParams)
             {
-                m_Params.hdrMode = VPHAL_HDR_MODE_H2H;
+                m_Params.uiMaxDisplayLum = surfOutput->pHDRParams->max_display_mastering_luminance;
+                if (surfOutput->pHDRParams->EOTF == VPHAL_HDR_EOTF_SMPTE_ST2084)
+                {
+                    m_Params.hdrMode = VPHAL_HDR_MODE_H2H;
+                }
             }
         }
-    }
-    else if (surfInput->pHDRParams->EOTF == VPHAL_HDR_EOTF_TRADITIONAL_GAMMA_SDR && surfOutput->pHDRParams->EOTF == VPHAL_HDR_EOTF_SMPTE_ST2084)
-    {
-        m_Params.hdrMode = VPHAL_HDR_MODE_INVERSE_TONE_MAPPING;
-    }
-
-    if (m_Params.hdrMode == VPHAL_HDR_MODE_NONE)
-    {
-        VP_PUBLIC_ASSERTMESSAGE("HDR Mode is NONE");
-        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+        else if (surfInput->pHDRParams->EOTF == VPHAL_HDR_EOTF_TRADITIONAL_GAMMA_SDR && surfOutput->pHDRParams->EOTF == VPHAL_HDR_EOTF_SMPTE_ST2084)
+        {
+            m_Params.hdrMode = VPHAL_HDR_MODE_INVERSE_TONE_MAPPING;
+        }
     }
 
     m_Params.pColorFillParams = params.pColorFillParams;
@@ -1676,6 +1705,20 @@ MOS_STATUS SwFilterBlending::Configure(VP_PIPELINE_PARAMS& params, bool isInputS
     m_Params.formatOutput   = surfInput->Format;
     m_Params.blendingParams = surfInput->pBlendingParams;
 
+    //Skip Blend PARTIAL for alpha input non alpha output
+    if (m_Params.blendingParams && m_Params.blendingParams->BlendType == BLEND_PARTIAL)
+    {
+        auto surfOutput = params.pTarget[0];
+        if (surfOutput)
+        {
+            if (IS_ALPHA_FORMAT(m_Params.formatInput) &&
+                !IS_ALPHA_FORMAT(surfOutput->Format))
+            {
+                VP_PUBLIC_NORMALMESSAGE("Force to use Blend Source instead of Blend Partial");
+                m_Params.blendingParams->BlendType = BLEND_SOURCE;
+            }
+        }
+    }
     return MOS_STATUS_SUCCESS;
 }
 
@@ -2100,6 +2143,211 @@ bool SwFilterCgc::IsBt2020ToRGB(VP_PIPELINE_PARAMS& params, bool isInputSurf, in
     return false;
 }
 
+/****************************************************************************************************/
+/*                                      SwFilterAiBase                                              */
+/****************************************************************************************************/
+
+SwFilterAiBase::SwFilterAiBase(VpInterface &vpInterface, FeatureType featureType) : SwFilter(vpInterface, featureType)
+{
+    m_Params.type = m_type;
+}
+
+SwFilterAiBase::~SwFilterAiBase()
+{
+    Clean();
+}
+
+MOS_STATUS SwFilterAiBase::Clean()
+{
+    VP_FUNC_CALL();
+
+    m_Params.settings.clear();
+    m_Params.splitGroupIndex.clear();
+    VP_PUBLIC_CHK_STATUS_RETURN(SwFilter::Clean());
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS SwFilterAiBase::Configure(VP_PIPELINE_PARAMS &params, bool isInputSurf, int surfIndex)
+{
+    VP_FUNC_CALL();
+
+    m_Params.settings.clear();
+    m_Params.splitGroupIndex.clear();
+    VP_PUBLIC_CHK_STATUS_RETURN(RegisterAiSettingPipe(params, m_Params.settings));
+    VP_PUBLIC_CHK_STATUS_RETURN(InitializeStageGroupIndex(m_Params.settings, m_Params.splitGroupIndex));
+    VP_PUBLIC_CHK_STATUS_RETURN(InitializeNpu(m_Params.settings));
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS SwFilterAiBase::InitializeStageGroupIndex(AI_SETTING_PIPE &settingPipe, AI_SPLIT_GROUP_INDEX &splitGroupIndex)
+{
+    splitGroupIndex.clear();
+    bool              customizedGroupIndex = false;
+    FEATURE_AI_ENGINE lastEngine           = FEATURE_AI_ENGINE::GPU;
+    for (uint32_t i = 0; i < settingPipe.size(); ++i)
+    {
+        auto &setting = settingPipe.at(i);
+        VP_PUBLIC_CHK_NULL_RETURN(setting);
+        if (setting->stageGroupIndex != 0)
+        {
+            customizedGroupIndex = true;
+            break;
+        }
+        if (i != 0 && setting->engine != lastEngine)
+        {
+            splitGroupIndex.push_back(i);
+        }
+        lastEngine = setting->engine;
+    }
+
+    if (customizedGroupIndex)
+    {
+        splitGroupIndex.clear();
+        uint32_t stageIndex = 0;
+        for (uint32_t i = 0; i < settingPipe.size(); ++i)
+        {
+            auto &setting = settingPipe.at(i);
+            VP_PUBLIC_CHK_NULL_RETURN(setting);
+            if (i == 0)
+            {
+                VP_PUBLIC_CHK_VALUE_RETURN(setting->stageGroupIndex, 0);
+            }
+            else
+            {
+                if (setting->stageGroupIndex == stageIndex)
+                {
+                    VP_PUBLIC_CHK_VALUE_RETURN(setting->engine, lastEngine);
+                }
+                else if (setting->stageGroupIndex == (stageIndex + 1))
+                {
+                    ++stageIndex;
+                    splitGroupIndex.push_back(i);
+                }
+                else
+                {
+                    VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+                }
+            }
+            lastEngine = setting->engine;
+        }
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS SwFilterAiBase::InitializeNpu(AI_SETTING_PIPE &settingPipe)
+{
+    bool hasNpu = false;
+    for (const auto& setting : settingPipe)
+    {
+        if (setting->engine == FEATURE_AI_ENGINE::NPU)
+        {
+            hasNpu = true;
+            break;
+        }
+    }
+    if (hasNpu)
+    {
+        VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface());
+        VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_osInterface);
+        VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_osInterface->npuInterface);
+        VP_PUBLIC_CHK_NULL_RETURN(m_vpInterface.GetHwInterface()->m_osInterface->npuInterface->pfnInit);
+        VP_PUBLIC_CHK_STATUS_RETURN(m_vpInterface.GetHwInterface()->m_osInterface->npuInterface->pfnInit(m_vpInterface.GetHwInterface()->m_osInterface->npuInterface));
+        m_gpuContxtOnHybridCmd = (1llu << static_cast<uint64_t>(MOS_GPU_CONTEXT_COMPUTE));
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
+FeatureParamAi &SwFilterAiBase::GetSwFilterParams()
+{
+    VP_FUNC_CALL();
+
+    return m_Params;
+}
+
+MOS_STATUS SwFilterAiBase::GetStagePipeSettings(uint32_t stageIndex, std::vector<AI_SINGLE_LAYER_BASE_SETTING *> &currentStagePipeSettings)
+{
+    currentStagePipeSettings.clear();
+    uint32_t startIndex = 0;
+    if (stageIndex != 0)
+    {
+        if ((stageIndex - 1) >= m_Params.splitGroupIndex.size())
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+        }
+        startIndex = m_Params.splitGroupIndex.at(stageIndex - 1);
+    }
+    uint32_t endIndex = stageIndex < m_Params.splitGroupIndex.size() ? m_Params.splitGroupIndex.at(stageIndex) : m_Params.settings.size();
+    if (startIndex >= endIndex)
+    {
+        VP_PUBLIC_ASSERTMESSAGE("Start Index of Kernel Pipe is Greater or Equal to End Index. The Execute Kernel Group will be Empty. Caused by Wrong Value in splitGroupIndex");
+        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+    }
+    for (uint32_t layerIndex = startIndex; layerIndex < endIndex; ++layerIndex)
+    {
+        if (layerIndex >= m_Params.settings.size())
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+        }
+        currentStagePipeSettings.push_back(m_Params.settings.at(layerIndex).get());
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
+SwFilter *SwFilterAiBase::Clone()
+{
+    VP_FUNC_CALL();
+
+    SwFilter *p = CreateSwFilter(m_type);
+
+    SwFilterAiBase *swFilter = dynamic_cast<SwFilterAiBase *>(p);
+    if (nullptr == swFilter)
+    {
+        DestroySwFilter(p);
+        return nullptr;
+    }
+
+    swFilter->m_Params.type            = m_Params.type;
+    swFilter->m_Params.splitGroupIndex = m_Params.splitGroupIndex;
+    swFilter->m_Params.stageIndex      = m_Params.stageIndex;
+    for (const auto& setting : m_Params.settings)
+    {
+        if (setting)
+        {
+            swFilter->m_Params.settings.push_back(setting->Clone());
+        }
+        else
+        {
+            VP_PUBLIC_ASSERTMESSAGE("Emptry setting pointer in settings pipe, will skip it");
+        }
+    }
+    return p;
+}
+
+bool SwFilterAiBase::operator==(SwFilter &swFilter)
+{
+    VP_FUNC_CALL();
+
+    SwFilterAiBase *p = dynamic_cast<SwFilterAiBase *>(&swFilter);
+    return nullptr != p && 0 == memcmp(&this->m_Params, &p->m_Params, sizeof(FeatureParamAi));
+}
+
+MOS_STATUS SwFilterAiBase::Update(VP_SURFACE *inputSurf, VP_SURFACE *outputSurf, SwFilterSubPipe &pipe)
+{
+    VP_FUNC_CALL();
+
+    VP_PUBLIC_CHK_NULL_RETURN(inputSurf);
+    VP_PUBLIC_CHK_NULL_RETURN(inputSurf->osSurface);
+    VP_PUBLIC_CHK_NULL_RETURN(outputSurf);
+    VP_PUBLIC_CHK_NULL_RETURN(outputSurf->osSurface);
+    m_Params.formatInput  = inputSurf->osSurface->Format;
+    m_Params.formatOutput = outputSurf->osSurface->Format;
+    return MOS_STATUS_SUCCESS;
+}
+
 /****************************************************************************************************/
 /*                                      SwFilterSet                                                 */
 /****************************************************************************************************/
@@ -2239,3 +2487,22 @@ RenderTargetType SwFilterSet::GetRenderTargetType()
     }
     return RenderTargetTypeParameter;
 }
+
+MOS_STATUS SwFilterSet::GetAiSwFilter(SwFilterAiBase*& swAiFilter)
+{
+    swAiFilter = nullptr;
+    for (auto& handle : m_swFilters)
+    {
+        SwFilterAiBase *filter = dynamic_cast<SwFilterAiBase *>(handle.second);
+        if (filter)
+        {
+            if (swAiFilter)
+            {
+                VP_PUBLIC_ASSERTMESSAGE("Only one AI Sw Filter is allowed in one SwFilterSet. More than one is found. Feature Types: %d and %d", swAiFilter->GetFeatureType(), filter->GetFeatureType());
+                VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+            }
+            swAiFilter = filter;
+        }
+    }
+    return MOS_STATUS_SUCCESS;
+}
\ No newline at end of file
