@@ -692,6 +692,11 @@ MOS_STATUS VpProcampReuse::UpdatePacket(SwFilter *filter, VpCmdPacket *packet)
 MOS_STATUS VpProcampReuse::UpdateFeatureParams(FeatureParamProcamp &params)
 {
     m_params = params;
+    if (params.procampParams)
+    {
+        m_procampParams        = *params.procampParams;
+        m_params.procampParams = &m_procampParams;
+    }
     return MOS_STATUS_SUCCESS;
 }
 
@@ -803,7 +808,7 @@ MOS_STATUS VpPacketReuseManager::PreparePacketPipeReuse(SwFilterPipe *&swFilterP
 
         if (m_pipeReused)
         {
-            m_packetPipeFactory.ReturnPacketPipe(m_pipeReused);
+            ReturnPacketPipeReused();
         }
 
         return MOS_STATUS_SUCCESS;
@@ -827,6 +832,10 @@ MOS_STATUS VpPacketReuseManager::PreparePacketPipeReuse(SwFilterPipe *&swFilterP
                 // unreused feature && nullptr != swfilter
                 m_reusable         = false;
                 isPacketPipeReused = false;
+                if (m_pipeReused)
+                {
+                    ReturnPacketPipeReused();
+                }
                 return MOS_STATUS_SUCCESS;
             }
             else
@@ -903,6 +912,12 @@ MOS_STATUS VpPacketReuseManager::PreparePacketPipeReuse(SwFilterPipe *&swFilterP
         auto scalingreuse = m_features.find(FeatureTypeScaling);
         auto cscreuse     = m_features.find(FeatureTypeCsc);
         auto rotreuse     = m_features.find(FeatureTypeRotMir);
+        if (scalingreuse == m_features.end() ||
+            cscreuse == m_features.end()     ||
+            rotreuse == m_features.end())
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_HANDLE);
+        }
 
         for (index = 0; index < m_pipeReused_TeamsPacket.size(); index++)
         {
@@ -933,35 +948,18 @@ MOS_STATUS VpPacketReuseManager::PreparePacketPipeReuse(SwFilterPipe *&swFilterP
 
             m_TeamsPacket_reuse = false;
 
-            foundPipe = false;
-            for (index = 0; index < m_pipeReused_TeamsPacket.size(); index++)
-            {
-                auto pipeReuseHandle = m_pipeReused_TeamsPacket.find(index);
-                if (pipeReuseHandle != m_pipeReused_TeamsPacket.end() &&
-                    m_pipeReused == pipeReuseHandle->second)
-                {
-                    foundPipe = true;
-                    break;
-                }
-            }
-
-            if (foundPipe == false)
-            {
-                if (m_pipeReused)
-                {
-                    m_packetPipeFactory.ReturnPacketPipe(m_pipeReused);
-                }
-            }
-            else
-            {
-                m_pipeReused = nullptr;
-            }
+            ReturnPacketPipeReused();
 
             return MOS_STATUS_SUCCESS;
         }
         else
         {
             auto pipe_TeamsPacket = m_pipeReused_TeamsPacket.find(index);
+            if (pipe_TeamsPacket == m_pipeReused_TeamsPacket.end())
+            {
+                VP_PUBLIC_ASSERTMESSAGE("Invalid teams packet pipe for reuse!");
+                VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+            }
 
             VpCmdPacket *packet = pipe_TeamsPacket->second->GetPacket(0);
             VP_PUBLIC_CHK_NULL_RETURN(packet);
@@ -999,29 +997,7 @@ MOS_STATUS VpPacketReuseManager::PreparePacketPipeReuse(SwFilterPipe *&swFilterP
         // m_pipeReused will be udpated in UpdatePacketPipeConfig.
         VP_PUBLIC_NORMALMESSAGE("Packet cannot be reused.");
 
-        foundPipe = false;
-        for (index = 0; index < m_pipeReused_TeamsPacket.size(); index++)
-        {
-            auto pipeReuseHandle = m_pipeReused_TeamsPacket.find(index);
-            if (pipeReuseHandle != m_pipeReused_TeamsPacket.end() &&
-                m_pipeReused == pipeReuseHandle->second)
-            {
-                foundPipe = true;
-                break;
-            }
-        }
-
-        if (foundPipe == false)
-        {
-            if (m_pipeReused)
-            {
-                m_packetPipeFactory.ReturnPacketPipe(m_pipeReused);
-            }
-        }
-        else
-        {
-            m_pipeReused = nullptr;
-        }
+        ReturnPacketPipeReused();
 
         return MOS_STATUS_SUCCESS;
     }
@@ -1091,6 +1067,13 @@ MOS_STATUS VpPacketReuseManager::UpdatePacketPipeConfig(PacketPipe *&pipe)
         return MOS_STATUS_SUCCESS;
     }
 
+    if (caps.enableSFCLinearOutputByTileConvert)
+    {
+        VP_PUBLIC_NORMALMESSAGE("Not reusable for enableSFCLinearOutputByTileConvert case.");
+        m_reusable = false;
+        return MOS_STATUS_SUCCESS;
+    }
+
     if (m_TeamsPacket && !m_TeamsPacket_reuse)
     {
         auto it = m_pipeReused_TeamsPacket.find(curIndex);
@@ -1111,10 +1094,7 @@ MOS_STATUS VpPacketReuseManager::UpdatePacketPipeConfig(PacketPipe *&pipe)
 
     if (!m_TeamsPacket)
     {
-        if (m_pipeReused)
-        {
-            m_packetPipeFactory.ReturnPacketPipe(m_pipeReused);
-        }
+        ReturnPacketPipeReused();
     }
 
     m_pipeReused = pipe;
@@ -1124,3 +1104,21 @@ MOS_STATUS VpPacketReuseManager::UpdatePacketPipeConfig(PacketPipe *&pipe)
     return MOS_STATUS_SUCCESS;
 }
 
+void VpPacketReuseManager::ReturnPacketPipeReused()
+{
+    VP_FUNC_CALL();
+    if (nullptr == m_pipeReused)
+    {
+        return;
+    }
+    for (const auto &pair : m_pipeReused_TeamsPacket)
+    {
+        if (pair.second == m_pipeReused)
+        {
+            m_pipeReused = nullptr;
+            return;
+        }
+    }
+    m_packetPipeFactory.ReturnPacketPipe(m_pipeReused);
+    return;
+}
